//
// ARF - Augmented Reality Framework (ETSI ISG ARF)
//
// Copyright 2022 ETSI
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Last change: June 2022
//

#define USING_OPENAPI_GENERATOR // alt. is Swagger

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using UnityEditor.Experimental.GraphView;
using UnityEngine.UIElements;
using UnityEditor.UIElements;
using ETSI.ARF.WorldStorage.REST;
using System;
using System.Threading.Tasks;
using Assets.ETSI.ARF.ARF_World_Storage_API.Editor.Windows;

#if USING_OPENAPI_GENERATOR
using Org.OpenAPITools.Api;
using Org.OpenAPITools.Model;
#else
using IO.Swagger.Api;
using IO.Swagger.Model;
#endif


//THIS CLASS IS GOING TO BE DELETED !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

namespace ETSI.ARF.WorldStorage.UI
{

    public class GraphWindow : EditorWindow
    {

        static public GraphWindow graphWindowSingleton;

        string tempStorageUrl;
        int tempStoragePort;

        [HideInInspector] public WorldStorageServer worldStorageServer;
        [HideInInspector] public WorldStorageUser worldStorageUser;

        [SerializeField] public List<string> trackables = new List<string>();

        private Color ori;
        private GUIStyle gsTest;

        private ARFGraphView myGraph;

        [MenuItem("ARFWorldStorage/Graph Editor")]
        public static void ShowWindow()
        {
            GetWindow<GraphWindow>("Menu Editor", true, typeof(SceneView));
            GraphWindow win = EditorWindow.GetWindow(typeof(GraphWindow), false, WorldStorageWindow.winName) as GraphWindow;
        }

        public GraphWindow()
        {
            graphWindowSingleton = this;
        }

        public void OnEnable()
        {
            Debug.Log("onEnable");
            //At unity launch
            if (SaveInfo.instance.nodePositions == null)
            {
                initNodePos();
            }

            tempStorageUrl = this.worldStorageServer.basePath;
            tempStoragePort = this.worldStorageServer.port;
            ConstructGraphView();
            rootVisualElement.Add(GenerateToolbar());
            myGraph.StretchToParentSize();
            //top offset so that the graph does'nt overlap with the rest of the ui
            myGraph.style.top = Length.Percent(11);
            rootVisualElement.Add(myGraph);
        }

        private void initNodePos()
        {

            SaveInfo.instance.nodePositions = new Dictionary<string, Rect>();
            foreach (Trackable track in TrackableRequest.GetAllTrackables(worldStorageServer))
            {
                if (track.KeyvalueTags.ContainsKey("unityAuthoringPosX") && track.KeyvalueTags.ContainsKey("unityAuthoringPosY"))
                {
                    var posX = float.Parse(track.KeyvalueTags["unityAuthoringPosX"][0]);
                    var posY = float.Parse(track.KeyvalueTags["unityAuthoringPosY"][0]);
                    Rect trackPos = new Rect(posX, posY, 135, 77);
                    SaveInfo.instance.nodePositions[track.UUID.ToString()] = trackPos;
                }
                else
                {
                    Rect trackPos = new Rect(0, 0, 135, 77);
                    SaveInfo.instance.nodePositions[track.UUID.ToString()] = trackPos;
                }
            }
            foreach (WorldAnchor wa in WorldAnchorRequest.GetAllWorldAnchors(worldStorageServer))
            {
                if (wa.KeyvalueTags.ContainsKey("unityAuthoringPosX") && wa.KeyvalueTags.ContainsKey("unityAuthoringPosY"))
                {
                    var posX = float.Parse(wa.KeyvalueTags["unityAuthoringPosX"][0]);
                    var posY = float.Parse(wa.KeyvalueTags["unityAuthoringPosY"][0]);
                    Rect waPos = new Rect(posX, posY, 135, 77);
                    SaveInfo.instance.nodePositions[wa.UUID.ToString()] = waPos;
                }
                else
                {
                    Rect trackPos = new Rect(0, 0, 135, 77);
                    SaveInfo.instance.nodePositions[wa.UUID.ToString()] = trackPos;
                }
            }
            SaveInfo.instance.toReFrame = true;
        }

        public void OnDisable()
        {
            if (positionshaveBeenChanged() && EditorUtility.DisplayDialog("Saving node positions", "The nodes positions have been changed, would you like to save them ?", "Yes", "No"))
            {
                SaveInfo.instance.nodePositions = myGraph.SavePositions();
                saveElemPositionsInServer();
                
            }
            //remove the toolbar
            rootVisualElement.RemoveAt(0);
            //remove the graph
            rootVisualElement.Remove(myGraph);

        }

        private void saveElemPositionsInServer()
        {
            List<Trackable> trackables = TrackableRequest.GetAllTrackables(worldStorageServer);
            List<WorldAnchor> anchors = WorldAnchorRequest.GetAllWorldAnchors(worldStorageServer);
            foreach (Trackable serverTrack in trackables)
            {
                if (serverTrack.KeyvalueTags.ContainsKey("unityAuthoringPosX") && serverTrack.KeyvalueTags.ContainsKey("unityAuthoringPosY"))
                {
                    float xLocal = SaveInfo.instance.nodePositions[serverTrack.UUID.ToString()].x;
                    float yLocal = SaveInfo.instance.nodePositions[serverTrack.UUID.ToString()].y;
                    float xServer = float.Parse(serverTrack.KeyvalueTags["unityAuthoringPosX"][0]);
                    float yServer = float.Parse(serverTrack.KeyvalueTags["unityAuthoringPosY"][0]);
                    if ((xLocal != xServer) && (yLocal != yServer))
                    {
                        var posX = new List<String>();
                        posX.Add(SaveInfo.instance.nodePositions[serverTrack.UUID.ToString()].x.ToString());
                        serverTrack.KeyvalueTags["unityAuthoringPosX"] = posX;
                        var posY = new List<String>();
                        posY.Add(SaveInfo.instance.nodePositions[serverTrack.UUID.ToString()].y.ToString());
                        serverTrack.KeyvalueTags["unityAuthoringPosY"] = posY;
                        TrackableRequest.UpdateTrackable(worldStorageServer, serverTrack);
                    }
                }
                else
                {
                    var posX = new List<String>();
                    posX.Add(SaveInfo.instance.nodePositions[serverTrack.UUID.ToString()].x.ToString());
                    serverTrack.KeyvalueTags["unityAuthoringPosX"] = posX;
                    var posY = new List<String>();
                    posY.Add(SaveInfo.instance.nodePositions[serverTrack.UUID.ToString()].y.ToString());
                    serverTrack.KeyvalueTags["unityAuthoringPosY"] = posY;
                    TrackableRequest.UpdateTrackable(worldStorageServer, serverTrack);
                }
            }
            foreach (WorldAnchor serverWorldAnchor in anchors)
            {
                if (serverWorldAnchor.KeyvalueTags.ContainsKey("unityAuthoringPosX") && serverWorldAnchor.KeyvalueTags.ContainsKey("unityAuthoringPosY"))
                {
                    float xLocal = SaveInfo.instance.nodePositions[serverWorldAnchor.UUID.ToString()].x;
                    float yLocal = SaveInfo.instance.nodePositions[serverWorldAnchor.UUID.ToString()].y;
                    float xServer = float.Parse(serverWorldAnchor.KeyvalueTags["unityAuthoringPosX"][0]);
                    float yServer = float.Parse(serverWorldAnchor.KeyvalueTags["unityAuthoringPosY"][0]);
                    if ((xLocal != xServer) && (yLocal != yServer))
                    {
                        var posX = new List<String>();
                        posX.Add(SaveInfo.instance.nodePositions[serverWorldAnchor.UUID.ToString()].x.ToString());
                        serverWorldAnchor.KeyvalueTags["unityAuthoringPosX"] = posX;
                        var posY = new List<String>();
                        posY.Add(SaveInfo.instance.nodePositions[serverWorldAnchor.UUID.ToString()].y.ToString());
                        serverWorldAnchor.KeyvalueTags["unityAuthoringPosY"] = posY;
                        WorldAnchorRequest.UpdateWorldAnchor(worldStorageServer, serverWorldAnchor);
                    }
                }
                else
                {
                    var posX = new List<String>();
                    posX.Add(SaveInfo.instance.nodePositions[serverWorldAnchor.UUID.ToString()].x.ToString());
                    serverWorldAnchor.KeyvalueTags["unityAuthoringPosX"] = posX;
                    var posY = new List<String>();
                    posY.Add(SaveInfo.instance.nodePositions[serverWorldAnchor.UUID.ToString()].y.ToString());
                    serverWorldAnchor.KeyvalueTags["unityAuthoringPosY"] = posY;
                    WorldAnchorRequest.UpdateWorldAnchor(worldStorageServer, serverWorldAnchor);
                }
            }
        }

        internal void createTrackableNode(string uUID, float posX, float posY)
        {
            Trackable track = TrackableRequest.GetTrackable(worldStorageServer, uUID);
            myGraph.createTrackableNode(track, posX, posY);
        }

        internal void createAnchorNode(string uUID, float posX, float posY)
        {
            WorldAnchor wa = WorldAnchorRequest.GetWorldAnchor(worldStorageServer, uUID);
            myGraph.createAnchorNode(wa, posX, posY);
        }

        internal void createLink(string uUID)
        {
            WorldLink worldLink = WorldLinkRequest.GetWorldLink(worldStorageServer, uUID);
            myGraph.createLink(worldLink);
        }

        private Toolbar GenerateToolbar()
        {
            var toolbar = new Toolbar();

            var createNodeT = new Button(clickEvent: () => {
                TrackableWindow.ShowWindow(worldStorageServer, worldStorageUser);
                TrackableWindow.winSingleton.nodePosX = 0;
                TrackableWindow.winSingleton.nodePosY = 0;
            });
            createNodeT.text = "Create Trackable";
            toolbar.Add(createNodeT);

            var createNodeWA = new Button(clickEvent: () => {
                WorldAnchorWindow.ShowWindow(worldStorageServer, worldStorageUser);
                WorldAnchorWindow.winSingleton.nodePosX = 0;
                WorldAnchorWindow.winSingleton.nodePosY = 0;
            });
            createNodeWA.text = "Create World Anchor";
            toolbar.Add(createNodeWA);

            toolbar.Add(new ToolbarSpacer()
            {
                flex = true
            });

            var reload = new Button(clickEvent: () => {
                Reload();
            })
            {
                text = "reload"
            };
            toolbar.Add(reload);

            return toolbar;
        }

        public void ConstructGraphView()
        {
                myGraph = new ARFGraphView
                {
                    name = "ARF Graph",
                    worldStorageServer = worldStorageServer,
                    worldStorageUser = worldStorageUser
                };
            myGraph.paintWorldStorage();
            SaveInfo.instance.toReFrame = true;
        }

        private void OnValidate()
        {
            Debug.Log("validate");
        }


        void OnGUI()
        {
            ori = GUI.backgroundColor; // remember ori color

            gsTest = new GUIStyle("window");
            gsTest.normal.textColor = WorldStorageWindow.arfColors[0];
            gsTest.fontStyle = FontStyle.Bold;

            EditorGUILayout.BeginVertical();
            EditorGUILayout.Space(24);


            EditorGUI.BeginChangeCheck();
            worldStorageServer = (WorldStorageServer)EditorGUILayout.ObjectField("World Storage Server", worldStorageServer, typeof(WorldStorageServer), false, GUILayout.Width(500));
            if (EditorGUI.EndChangeCheck())
            {
                OnDisable();
                initNodePos();
                OnEnable();
                //ConstructGraphView();
            }


            /*EditorGUILayout.BeginHorizontal();
            tempStorageUrl = EditorGUILayout.TextField(tempStorageUrl);
            EditorGUILayout.LabelField(":", GUILayout.ExpandWidth(false), GUILayout.Width(5));
            tempStoragePort = EditorGUILayout.IntField(tempStoragePort, GUILayout.ExpandWidth(false), GUILayout.Width(40));
            GUILayout.Space(5);
           *//* if (GUILayout.Button("Change WorldStorage"))
            {
                ChangeWorldStorage();
            }*//*
            GUILayout.FlexibleSpace();
            EditorGUILayout.EndHorizontal();*/


            GUI.contentColor = WorldStorageWindow.arfColors[1];
            WorldStorageWindow.DrawCopyright();
            if (SaveInfo.instance.toReFrame)
            {
                myGraph.FrameAllElements();
                SaveInfo.instance.toReFrame = false;
            }
            EditorGUILayout.EndVertical();
        }

        private void ChangeWorldStorage()
        {
            string[] guids1 = AssetDatabase.FindAssets("t:WorldStorageServer", null);

            foreach (string guid1 in guids1)
            {
                string path = AssetDatabase.GUIDToAssetPath(guid1);
                WorldStorageServer server = AssetDatabase.LoadAssetAtPath(path, typeof(WorldStorageServer)) as WorldStorageServer;
                if ((server.port == tempStoragePort)&&(server.basePath == tempStorageUrl))
                {
                    this.worldStorageServer = server;
                    myGraph.worldStorageServer = server;
                }
                else
                {
                    WorldStorageServer newServer = (WorldStorageServer) ScriptableObject.CreateInstance(typeof(WorldStorageServer));
                    newServer.basePath = tempStorageUrl;
                }
            }
        }

        void Update()
        {
        }

        private bool positionshaveBeenChanged()
        {
            bool changed = false;
            foreach(ARFNode node in myGraph.nodes)
            {
                float nodeX = node.GetPosition().x;
                float nodeY = node.GetPosition().y;
                float dataX = SaveInfo.instance.nodePositions[node.GUID].x;
                float dataY = SaveInfo.instance.nodePositions[node.GUID].y;
                if((nodeX != dataX) || (nodeY != dataY))
                {
                    changed = true;
                    return changed;
                }
            }
            return changed;
        }

        public void Reload()
        {
            if (positionshaveBeenChanged() && EditorUtility.DisplayDialog("Saving node positions", "The nodes positions have been changed, would you like to save them ?", "Yes", "No"))
            {
                SaveInfo.instance.nodePositions = myGraph.SavePositions();
            }
            myGraph.reload();
            SaveInfo.instance.toReFrame = true;
        }

        public void RefreshWithoutReframe()
        {
            if (positionshaveBeenChanged() && EditorUtility.DisplayDialog("Saving node positions", "The nodes positions have been changed, would you like to save them ?", "Yes", "No"))
            {
                SaveInfo.instance.nodePositions = myGraph.SavePositions();
            }
            myGraph.reload();
        }
    }
}